@@ -193,6 +193,40 @@ get_gdk_color_from_rgb (GdkColor *c, guint32 rgb)
 	c->pixel = 0;
 }
 
+static void
+menu_position_func (GtkMenu           *menu,
+                    int               *x,
+                    int               *y,
+                    gboolean          *push_in,
+                    GreeterItemInfo *item)
+{
+	GtkAllocation rect;
+	GtkRequisition  requisition;
+
+	rect = item->allocation; 
+	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+	*x = rect.x;
+	if (requisition.height <= rect.y)
+		*y = rect.y - requisition.height;
+	else
+		*y = rect.y + rect.height;
+
+	*push_in = TRUE;
+}
+
+/*
+ * The button with this handler never appears in the F10 menu, so
+ * it can make use of callback data.
+ */
+static void
+greeter_options_handler (GreeterItemInfo *item, GtkWidget *menubar)
+{
+	gtk_menu_popup (GTK_MENU(gtk_menu_item_get_submenu(
+		        gtk_container_get_children(GTK_CONTAINER(menubar))->data)), 
+			NULL, NULL, (GtkMenuPositionFunc)menu_position_func, 
+			item, 0, gtk_get_current_event_time());
+}
+
 void
 greeter_item_create_canvas_item (GreeterItemInfo *item)
 {
@@ -387,7 +421,11 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 				   "height", (double)rect.height,
 				   "width", (double)rect.width,
 				   NULL);
-
+	    
+	    greeter_item_register_action_callback ("options_button",
+						   (ActionFunc)greeter_options_handler,
+						   menubar);
+	    
 	    /* Here add a tooltip, so that the user knows about F10 */
 	    tooltips = gtk_tooltips_new ();
 	    gtk_tooltips_set_tip (tooltips, GTK_WIDGET (entry),
@@ -408,7 +446,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 					NULL);
 
     /* cursor blinking is evil on remote displays, don't do it forever */
-    gdm_setup_blinking_entry (entry);
+    gdm_common_setup_blinking_entry (entry);
 
     break;
 
@@ -607,7 +645,7 @@ greeter_canvas_item_break_set_string (GreeterItemInfo *info,
 			while (i < n_chars) {
 				g_string_append_unichar (word, ch);
 				p = g_utf8_next_char (p);
-				i ++;
+				i++;
 
 				if (ch == '>') {
 					ch = g_utf8_get_char (p);
@@ -625,12 +663,12 @@ greeter_canvas_item_break_set_string (GreeterItemInfo *info,
 				in_current_row = 0;
 		}
 
-		in_current_row ++;
+		in_current_row++;
 		g_string_append_unichar (word, ch);
 
 		p = g_utf8_next_char (p);
-		i ++;
-		ia ++;
+		i++;
+		ia++;
 
 		/* eeek! */
 		if (ia >= n_attrs) {
@@ -638,7 +676,7 @@ greeter_canvas_item_break_set_string (GreeterItemInfo *info,
 				ch = g_utf8_get_char (p);
 				g_string_append_unichar (word, ch);
 				p = g_utf8_next_char (p);
-				i ++;
+				i++;
 			}
 			break;
 		}
